<?php
namespace BrewPixel\Barista\Bin\Composer;

use Composer\Script\Event;
use Composer\Command\ShowCommand;
use BrewPixel\Barista\Percolator\Validate;
use BrewPixel\Barista\Percolator\Sanitise;
use LayerShifter\TLDExtract\Extract;

/**
 * Class PostCreateProjectScript
 * @package BrewPixel\Roastery
 */
class PostCreateProjectScript {

	public static function bashInstall( Event $event ) {

		$io = $event->getIO();
		$project_root   = dirname( __DIR__ . DIRECTORY_SEPARATOR . '../../../../../..' );

		$choice                        = $io->select(
			"\nWhat would you like to do?\n",
			array(
				0 => 'New Project: <info>WordPress Genesis-based</info>',
				1 => 'New Project: <info>WordPress Genesis-based WooCommerce</info>',
				2 => 'New Project: <info>WordPress Moka based</info>',
				3 => 'New Project: <info>WordPress Moka based WooCommerce</info>',
				4 => 'Existing Project: <info>Install/Re-install WordPress</info> <comment>Coming soon</comment>',
				5 => 'Maintenance: <info>Update Core/Plugins/Themes/Languages</info> <comment>Coming soon</comment>',
				6 => 'Advanced: <comment>Coming soon</comment>',
				7 => 'List all BrewPixel BitBucket Repos: <comment>Needs cleaned and paginated.</comment>',
			),
			'New Project: <info>WordPress Genesis-based</info>',
			false,
			'Value "%s" is invalid',
			false
		);

		$project_new_wordpress_genesis = '0';
		$project_new_wordpress_genesis_woocommerce = '1';
		$project_new_wordpress_moka = '2';
		$project_new_wordpress_moka_woocommerce = '3';
		$install_reinstall_wordpress_current_project = '4';
		$maintenance_update_core_plugins_themes_languages = '5';
		$advanced = '6';
		$list_repos = '7';

		if ( $list_repos === $choice ) {
			$bitbucket_consumer_key    = $io->askAndHideAnswer( '<info>Bitbucket consumer key</info>: ' );
			$bitbucket_consumer_secret = $io->askAndHideAnswer( '<info>Bitbucket consumer secret</info>: ' );
			$bitbucket_team_name = 'brewpixel';
			putenv( "BITBUCKET_CONSUMER_KEY=$bitbucket_consumer_key" );
			putenv( "BITBUCKET_CONSUMER_SECRET=$bitbucket_consumer_secret" );
			putenv( "BITBUCKET_TEAM_NAME=$bitbucket_team_name" );
			$list_bitbucket_repos = __DIR__ . '/list_bitbucket_repos.sh';
			exec( "bash $list_bitbucket_repos", $configoutput, $wp_config_result );
			print_r( $configoutput );
			var_dump( $wp_config_result );
		}
		if ( $install_reinstall_wordpress_current_project === $choice ) {
			$client_name = $io->askAndValidate(
				'<info>Client company name</info>: ',
				array( __CLASS__, 'validate_client_name' ),
				null,
				null
			);
			$main_repo_name           = $io->askAndValidate(
				'<info>Domain name of website</info>: ',
				array( __CLASS__, 'validateURLwithoutProtocol' ),
				null,
				null
			);
			$theme_name     = $io->askAndValidate(
				'<info>New WordPress theme name</info>: ',
				array( __CLASS__, 'validate_theme_name' ),
				null,
				null
			);
			$theme_name_lowercase_hyphen_no_spaces = self::validate_lowercase_hyphen_no_spaces( $theme_name );
			$db_name = self::default_db_name( $main_repo_name );
			$db_user = 'root';
			$db_password = 'root';
			$db_host = 'localhost';
			$bitbucket_team_name = 'brewpixel';
			$bitbucket_project_name = $main_repo_name;
			$bitbucket_project_key = strtoupper( substr( $bitbucket_project_name, 0, 3 ) );
			$bitbucket_project_description = "$main_repo_name project.";
			$extract = new Extract;
			$domain_extract = $extract->parse( $main_repo_name );
			$domain_without_tld = preg_replace( '/\.' . $domain_extract['suffix'] . '$/', '', $domain_extract );

			$site_url = 'http://' . $domain_without_tld . '.local';
			$site_title = 'My site';
			$blog_name = 'My blog';
			$blog_description = 'A WordPress site by BrewPixel';
			$composer_project = 1;
			$ignore_themes = '1';
			$ignore_plugins = '1';
			$ignore_wp = 1;
			$all_fake_content = 1;
			$fake_comments    = 1;
			$fake_posts       = 1;
			$fake_client_user = 1;
			$development_mode = '1';

			putenv( "CHOICE=$choice" );
			putenv( "BITBUCKET_TEAM_NAME=$bitbucket_team_name" );
			putenv( "BITBUCKET_PROJECT_NAME=$bitbucket_project_name" );
			putenv( "BITBUCKET_PROJECT_KEY=$bitbucket_project_key" );
			putenv( "BITBUCKET_PROJECT_DESCRIPTION=$bitbucket_project_description" );
			putenv( "CLIENT_NAME=$client_name" );
			putenv( "MAIN_REPO_NAME=$main_repo_name" );
			putenv( "THEME_NAME=$theme_name" );
			putenv( "THEME_NAME_LOWERCASE_HYPHEN_NO_SPACES=$theme_name_lowercase_hyphen_no_spaces" );
			putenv( "DB_NAME=$db_name" );
			putenv( "DB_USER=$db_user" );
			putenv( "DB_PASSWORD=$db_password" );
			putenv( "DB_HOST=$db_host" );
			putenv( "SITE_URL=$site_url" );
			putenv( "SITE_TITLE=$site_title" );
			putenv( "BLOG_NAME=$blog_name" );
			putenv( "BLOG_DESCRIPTION=$blog_description" );
			putenv( "COMPOSER_PROJECT=$composer_project" );
			putenv( "IGNORE_THEMES=$ignore_themes" );
			putenv( "IGNORE_PLUGINS=$ignore_plugins" );
			putenv( "IGNORE_WP=$ignore_wp" );
			putenv( "ALL_FAKE_CONTENT=$all_fake_content" );
			putenv( "FAKE_COMMENTS=$fake_comments" );
			putenv( "FAKE_POSTS=$fake_posts" );
			putenv( "FAKE_CLIENT_USER=$fake_client_user" );
			putenv( "DEVELOPMENT_MODE=$development_mode" );

			$wp_create_config = __DIR__ . '/../wp/create_wordpress_config';
			exec( "bash $wp_create_config", $output, $wp_result );
			$wp_install_check = __DIR__ . '/../wp/install_wordpress_check.sh';
			exec( "bash $wp_install_check", $output, $wp_result );

			if ( 1 === $output ) {
				$wp = __DIR__ . '/../wp/install_wordpress';
				exec( "bash $wp", $output, $wpout );
			} elseif ( 1 === $wp_result ) {
				die( 'There was an error. Likely we can not connect to the database' );
			} else {
				$database_drop                 = $io->askConfirmation( '<info>The database already exists. Can we delete it?</info> ' );
				if ( true === $database_drop ) {
					$wp = __DIR__ . '/../wp/install_wordpress';
					exec( "bash $wp", $installoutput, $wpout );
				} else {
					die( 'You did not want to delete the database so we did not begin the WordPress installation.' );
				}
			}

			echo 'finished';
		}

		if ( $maintenance_update_core_plugins_themes_languages === $choice ) {
			/*
			 * Variables to incorporate into a script.
			 *
			 *  WordPress path: public_html/wp
				WordPress as a submodule: YES
				Other submodules: public_html/content/plugins/brewpress
				Locale

			@TODO: Save a report of updates.
			@TODO: Add no auto updates to wp-config.php
			@TODO: Add deploy methods git to SFTP etc.
			@TODO: Build script system to get current production database and bring into local and staging
			-
			Make sure you have a local copy of the website and it is set-up in your local development environment.

			Open up your local dev environment. Is it vagrant?

				vagrant up

			SSH into it if required such as for a vagrant box.

                ssh vagrant@

			Check what git branch it is on

                git branch -a

			Are there any files modified that should be committed or discarded?

				git status

			Yes, there are files that are modified and should be committed.

                git add .

                git commit -m 'Message'

			No, there aren't. Proceed to next step.

			Does the branch have commits different to master? Check if commmit hashes are the same. Check if git master is up to date with origin.

			On production:

                git log -1

			Full example:

			Login to the production server and see if there are updates to be committed or downloaded.

			Maybe create a new "update-2016-05-01" branch from master?

			    git checkout -b maintenance/update-2016-06-02 origin/master
			Checklist:

			1. Ensure website files and database have recently been backed up
			    - Log in to backup server `ssh [backup_user]@[backup_server_ip]`
			    - `cd /home/[backup_user]/backup/archive/daily`
			    - Then cd into directory with the same day of the week as today to check that the backup is there.
			    - To make a manual backup of files `tar -czf /home/goldie/backups/initial_backup.tar.gz /srv/www/sonobex.com/www`
			    - @todo Check database
			    - To make a manual backup of all databases run `for I in $(mysql -uroot -e 'show databases' -s --skip-column-names); do mysqldump -uroot $I | gzip > "/home/goldie/mysqldump/$I.`date +\%Y-\%m-\%d`.sql.gz"; done` as root user. Or individually ` mysqldump -uroot sonobexcom | gzip > "/home/goldie/mysqldump/sonobexcom.`date +\%Y-\%m-\%d`.sql.gz"`
			2. On local development system `git checkout develop` or `git checkout -b update-2016-05-01` for example
			3. Update WordPress core:

			Script: (Update WordPress core on local files and not within Vagrant virtual machine unless you have permission rights set-up there.)

			@TODO: We may want to do a puppet run before hand or at least check that puppet has run that day.
			@TODO: Be aware of what user is running each command. Does it need sudo? Likely best to run most commands as the user for that website.
			@TODO: Example sudo -u pcginvestcoukstaging -i
			@TODO: For the end report we will want to know the list of plugins installed/active etc as well as what was updated. This way we can look at caching plugins or inactive etc.
			cd wp
		    git fetch
		    git fetch --tags
		    # If you want to check the latest tags
		    git tags
		    # Replace number with latest tag
		    git checkout 3.4
		    cd ../
		    git add wp
		    git commit -m 'Update to WordPress 3.4'
		    git push

			If "classic" WordPress set-up with wp-content, wp-admin, wp-includes in public directory:
				wp core update
			git add -A
			git commit -m 'Update WordPress'
				wp plugin update --all
			git add -A public_html/wp-content/plugins/wordpress-seo
			git add -A public_html/wp-content/plugins/other-plugins etc
				wp theme update --all
				wp core language update

			*/
		}

		if ( $project_new_wordpress_genesis === $choice ||
		     $project_new_wordpress_genesis_woocommerce === $choice ||
		     $project_new_wordpress_moka === $choice ||
		     $project_new_wordpress_moka_woocommerce === $choice
		) {
			echo "Installing Genesis requirements\n";
			exec( "cd $project_root && composer require \
			wpackagist-plugin/log-deprecated-notices:* \
			wpackagist-plugin/debug-bar:* \
			wpackagist-plugin/debug-bar-console:* \
			wpackagist-plugin/debug-bar-extender:* \
			wpackagist-plugin/debug-bar-shortcodes:* \
			wpackagist-plugin/debug-bar-constants:* \
			wpackagist-plugin/debug-bar-post-types:* \
			wpackagist-plugin/debug-bar-cron:* \
			wpackagist-plugin/debug-bar-actions-and-filters-addon:* \
			wpackagist-plugin/debug-bar-transients:* \
			wpackagist-plugin/debug-bar-list-dependencies:* \
			wpackagist-plugin/debug-bar-remote-requests:dev-trunk \
			wpackagist-plugin/developer:* \
			wpackagist-plugin/user-switching:* \
			--dev --no-update --no-interaction" );

			exec( "cd $project_root && composer require \
			johnpbloch/wordpress:'>=4.5.3' \
			brewpixel/brewpress:dev-master \
			wpackagist-plugin/akismet:dev-trunk \
			--no-update --no-interaction");

			putenv( 'CHILD_THEME=true' );
		}

		if ( $project_new_wordpress_genesis === $choice ||
		     $project_new_wordpress_genesis_woocommerce === $choice
		) {
			exec( "cd $project_root && composer require \
			brewpixel/genesis:* \
			brewpixel/genesis-child-starter:dev-master \
			--no-update --no-interaction" );
		}

		if ( $project_new_wordpress_moka === $choice ||
		     $project_new_wordpress_moka_woocommerce === $choice
		) {
			exec( "cd $project_root && composer require \
			brewpixel/moka:^0.1.0 \
			brewpixel/moka-child-starter:dev-master \
			--no-update --no-interaction" );
		}

		if ( $project_new_wordpress_genesis_woocommerce === $choice ||
		     $project_new_wordpress_moka_woocommerce === $choice
		) {
			exec( "cd $project_root && composer require \
			wpackagist-plugin/woocommerce:dev-trunk \
			--no-update --no-interaction" );
		}

		if ( $project_new_wordpress_genesis === $choice ||
		     $project_new_wordpress_genesis_woocommerce === $choice ||
		     $project_new_wordpress_moka === $choice ||
		     $project_new_wordpress_moka_woocommerce === $choice ) {
			exec( "cd $project_root && composer update" );
			// language
			// choose plugins
			// import database
			$simple = $io->askConfirmation( '<info>Simple default install?</info> [<comment>y,n</comment>]? ' );
			if ( true === $simple ) {
				echo "Ok we'll ask as few questions as possible.\n";
			} else {
				echo "Ok, you're in control.";
			}
			$client_name = $io->askAndValidate(
				'<info>Client company name</info>: ',
				array( __CLASS__, 'validate_client_name' ),
				null,
				null
			);
			$main_repo_name = $io->askAndValidate(
				'<info>Domain name of website</info>: ',
				array( __CLASS__, 'validateURLwithoutProtocol' ),
				null,
				null
			);
			$theme_name     = $io->askAndValidate(
				'<info>New WordPress theme name</info>: ',
				array( __CLASS__, 'validate_theme_name' ),
				null,
				null
			);
			$theme_name_lowercase_hyphen_no_spaces = self::validate_lowercase_hyphen_no_spaces( $theme_name );
			if ( $simple ) {
				$db_name                       = self::default_db_name( $main_repo_name );
				$db_user                       = 'root';
				$db_password                   = 'root';
				$db_host                       = 'localhost';
				$bitbucket_team_name           = 'brewpixel';
				$bitbucket_project_name        = $main_repo_name;
				$bitbucket_project_key         = strtoupper( substr( $bitbucket_project_name,
				0, 3 ) );
				$bitbucket_project_description = "$main_repo_name project.";
				$extract                       = new Extract;
				$domain_extract                = $extract->parse( $main_repo_name );
				$domain_without_tld            = preg_replace( '/\.' . $domain_extract['suffix'] . '$/',
				'', $domain_extract );

				$site_url         = 'http://' . $domain_without_tld . '.local';
				$site_title       = 'My site';
				$blog_name        = 'My blog';
				$blog_description = 'A WordPress site by BrewPixel';
				$composer_project = 1;
				$ignore_themes    = '1';
				$ignore_plugins   = '1';
				$ignore_wp        = 1;
				$all_fake_content = 1;
				$fake_comments    = 1;
				$fake_posts       = 1;
				$fake_client_user = 1;
				$development_mode = '1';
			} else {
				$db_name                = $io->askAndValidate(
					'<info>Database name</info>: ',
					array( __CLASS__, 'validate_db_name' ),
					null,
					self::default_db_name( $main_repo_name )
				);
				$db_user                = $io->askAndValidate(
					'<info>Database user</info>: ',
					array( __CLASS__, 'validate_db_username' ),
					null,
					'root'
				);
				$db_password            = $io->askAndHideAnswer( '<info>Database password</info>: ' );
				$db_host                = $io->askAndValidate(
					'<info>Database host</info> (<comment>Default: localhost</comment>): ',
					array( __CLASS__, 'validate_lowercase_no_spaces' ),
					null,
					'localhost'
				);
				$bitbucket_team_name    = $io->askAndValidate(
					'<info>Bitbucket team name</info> (<comment>Default: brewpixel</comment>): ',
					array( __CLASS__, 'validate_lowercase_no_spaces' ),
					null,
					'brewpixel'
				);
				$bitbucket_project_name = $io->ask( '<info>Bitbucket project name</info>: ' );

				$bitbucket_project_key         = $io->ask( '<info>Bitbucket project key</info>: ' );
				$bitbucket_project_description = $io->ask( '<info>Bitbucket project description</info>: ' );
				$site_url                      = $io->ask( '<info>Local development site URL</info>: ' );
				$site_title                    = $io->ask( '<info>Site title</info> (<comment>You can leave it blank for the default</comment>): ' );
				$blog_name                     = $io->askAndValidate(
					'<info>Blog name</info> (<comment>You can leave it blank for the default</comment>): ',
					array( __CLASS__, 'validate_lowercase_no_spaces' ),
					null,
					'brewpixel'
				);
				$blog_description              = $io->ask( '<info>Blog description</info> (<comment>You can leave it blank for the default</comment>): ' );
				$composer_project              = $io->askConfirmation( '<info>Is this a composer project?</info> [<comment>y,n</comment>]? ' );
				if ( true === $composer_project ) {
					$ignore_themes  = $io->askConfirmation( "<info>Since this is a composer project we recommend to git ignore the WordPress themes. Is that ok?</info> [<comment>y,n</comment>]? \n(<comment>You can leave it blank for the default</comment>) " );
					$ignore_plugins = $io->askConfirmation( "<info>Since this is a composer project we recommend to git ignore the WordPress plugins. Is that ok?</info> [<comment>y,n</comment>]? \n(<comment>You can leave it blank for the default</comment>) " );
					$ignore_wp      = $io->askConfirmation( "<info>Since this is a composer project we recommend to git ignore the WordPress core. Is that ok?</info> [<comment>y,n</comment>]? \n(<comment>You can leave it blank for the default</comment>) " );
				} else {
					$ignore_themes  = $io->askConfirmation( '<info>Git ignore WordPress themes?</info> [<comment>y,n</comment>]? (<comment>You can leave it blank for the default</comment>) ' );
					$ignore_plugins = $io->askConfirmation( '<info>Git ignore WordPress plugins?</info> [<comment>y,n</comment>]? (<comment>You can leave it blank for the default</comment>) ' );
					$ignore_wp      = $io->askConfirmation( '<info>Git ignore WordPress core?</info> [<comment>y,n</comment>]? (<comment>You can leave it blank for the default</comment>) ' );
				}

				$all_fake_content = $io->askConfirmation( '<info>All fake content?</info> [<comment>y,n</comment>]? ' );

				if ( true === $all_fake_content ) {
					$fake_comments    = 1;
					$fake_posts       = 1;
					$fake_client_user = 1;
				} else {
					$fake_comments    = $io->askConfirmation( '<info>Fake comments? </info> [<comment>y,n</comment>]? ' );
					$fake_posts       = $io->askConfirmation( '<info>Fake posts? </info> [<comment>y,n</comment>]? ' );
					$fake_client_user = $io->askConfirmation( '<info>Fake_client_user? </info> [<comment>y,n</comment>]? ' );
				}

				$development_mode = $io->askConfirmation( '<info>Development mode? </info> [<comment>y,n</comment>]? ' );

				// $result = preg_replace("/(^\\S+/{1}.+$)(?m)/uiUm", "hello $1", $searchText);
				// print_r($outputArray);
				// self::showPackages( $show );
				// Capture these into a list and ask if any are to be removed:
				// composer remove vendor/package vendor/package2
				$packages = [ 999 ];
				while ( 0 < $packages[0] && '' !== $packages[0] ) {

					exec( 'composer show --self', $array_packages,
					$return_value );

					$filtered_packages = preg_grep( '#(^\S+/{1}.+$)(?m)#',
					$array_packages );
					$root_packages     = array_merge( array( 'No' ),
					array_values( $filtered_packages ) );
					unset( $packages );
					$packages = $io->select(
						"<info>Select any packages you would like to remove:</info>\n<comment>(Multi-select comma separated numbers. Example 1,2,3)</comment> ",
						$root_packages,
						'No',
						false,
						'Value "%s" is invalid',
						true
					);

					if ( '0' !== $packages[0] && 'No' !== $packages[0] && '0' !== $packages && 'No' !== $packages ) {
						if ( '0' === $packages || 'No' === $packages || 'n' === $packages || '0' === $packages[0] || 'No' === $packages[0] || 'n' === $packages[0] ) {
							echo 'Exiting';
						} else {
							foreach ( $packages as $package ) {
								$package_remove = preg_replace( '/(^.*)\\s.*$/uiUm',
									'$1',
								$root_packages[ $package ] );
								exec( "cd $project_root && composer remove $package_remove --no-update --no-interaction" );
							}
						}
					}
					unset( $array_packages, $root_packages, $filtered_packages );
				}

				// Ask what packages they would like to add to dev-only:
				// composer require vendor/package:2.* vendor/package2:dev-master --dev --no-update
				$package = '';
				while ( 'No' !== $package && 'n' !== $package && 0 !== $package ) {
					$package = $io->askAndValidate(
						"<info>Add a package to use for development:</info>\n<comment>(This will be added to require-dev. Example vendor/package:dev-master)</comment> ",
						array( __CLASS__, 'validatePackage' ),
						null,
						'No'
					);

					if ( '0' === $package || 'No' === $package || 'n' === $package ) {
						echo 'Exiting';
					} else {
						exec( "cd $project_root && composer require $package --dev --no-update --no-interaction" );
					}
				}
				unset( $package );

				// Ask what packages they would like to add for production as well:
				// composer require vendor/package:2.* vendor/package2:dev-master --no-update
				$package = '';
				while ( 'No' !== $package && 'n' !== $package && 0 !== $package ) {
					$package = $io->askAndValidate(
						"<info>Add a package to use for all environments:</info>\n<comment>(This will be added to require. Example vendor/package:dev-master)</comment> ",
						array( __CLASS__, 'validatePackage' ),
						null,
						'No'
					);

					if ( '0' === $package || 'No' === $package || 'n' === $package ) {
						echo 'Exiting';
					} else {
						exec( "cd $project_root && composer require $package --no-update --no-interaction" );
					}
				}

				// Ask if they want to search for any:
				// composer search monolog
			}

			$bitbucket_consumer_key    = $io->askAndHideAnswer( '<info>Bitbucket consumer key</info>: ' );
			$bitbucket_consumer_secret = $io->askAndHideAnswer( '<info>Bitbucket consumer secret</info>: ' );

			/* TESTING / DEBUG */
			echo "$choice\n";
			echo "$main_repo_name\n";
			echo "$theme_name\n";
			echo "$theme_name_lowercase_hyphen_no_spaces\n";
			echo "$db_name\n";
			echo "$db_user\n";
			echo "$db_password\n";
			echo "$db_host\n";
			echo "$bitbucket_consumer_key\n";
			echo "$bitbucket_consumer_secret\n";
			echo "$bitbucket_team_name\n";
			echo "$bitbucket_project_name\n";
			echo "$bitbucket_project_key\n";
			echo "$bitbucket_project_description\n";
			echo "$site_url\n";
			echo "$site_title\n";
			echo "$blog_name\n";
			echo "$blog_description\n";
			echo "$composer_project\n";
			echo "$ignore_themes\n";
			echo "$ignore_plugins\n";
			echo "$ignore_wp\n";
			echo "$all_fake_content\n";
			echo "$fake_comments\n";
			echo "$fake_posts\n";
			echo "$fake_client_user\n";
			echo "$development_mode\n";
			// About to perform the following:
			// * Halt Vagrant (if running)
			// * Create directory selbycarpetsnew in /Users/goldie/Sites/vagrant-local/www
			// * Create files vvv-init.sh, wp-cli.yml, and vvv-hosts in directory selbycarpetsnew
			// * Create file selbycarpetsnew.conf in /Users/goldie/Sites/vagrant-local/config/nginx-config/sites
			// * Run `vagrant up --provision` to initialize site
			// Provisioning Vagrant will do the following:
			// * Create database selbycarpetsnew
			// * Install WordPress (release version) in the htdocs directory
			// * Make the site visible at selbycarpetsnew.local
			// Continue (y/n)? : y
			// @TODO Let's save a lot of this info like the consumer key in a database and then we can confirm rather than ask.
			// If the database is not available then we can fallback to asking.
			putenv( "CHOICE=$choice" );
			putenv( "BITBUCKET_CONSUMER_KEY=$bitbucket_consumer_key" );
			putenv( "BITBUCKET_CONSUMER_SECRET=$bitbucket_consumer_secret" );
			putenv( "BITBUCKET_TEAM_NAME=$bitbucket_team_name" );
			putenv( "BITBUCKET_PROJECT_NAME=$bitbucket_project_name" );
			putenv( "BITBUCKET_PROJECT_KEY=$bitbucket_project_key" );
			putenv( "BITBUCKET_PROJECT_DESCRIPTION=$bitbucket_project_description" );
			putenv( "CLIENT_NAME=$client_name" );
			putenv( "MAIN_REPO_NAME=$main_repo_name" );
			putenv( "THEME_NAME=$theme_name" );
			putenv( "THEME_NAME_LOWERCASE_HYPHEN_NO_SPACES=$theme_name_lowercase_hyphen_no_spaces" );
			putenv( "DB_NAME=$db_name" );
			putenv( "DB_USER=$db_user" );
			putenv( "DB_PASSWORD=$db_password" );
			putenv( "DB_HOST=$db_host" );
			putenv( "SITE_URL=$site_url" );
			putenv( "SITE_TITLE=$site_title" );
			putenv( "BLOG_NAME=$blog_name" );
			putenv( "BLOG_DESCRIPTION=$blog_description" );
			putenv( "COMPOSER_PROJECT=$composer_project" );
			putenv( "IGNORE_THEMES=$ignore_themes" );
			putenv( "IGNORE_PLUGINS=$ignore_plugins" );
			putenv( "IGNORE_WP=$ignore_wp" );
			putenv( "ALL_FAKE_CONTENT=$all_fake_content" );
			putenv( "FAKE_COMMENTS=$fake_comments" );
			putenv( "FAKE_POSTS=$fake_posts" );
			putenv( "FAKE_CLIENT_USER=$fake_client_user" );
			putenv( "DEVELOPMENT_MODE=$development_mode" );

			exec( "cd $project_root && composer update" );

			$install = __DIR__ . '/post_create_project_script.sh';
			echo 'about to install';

			exec( "bash $install", $output, $return_value );

			$wp_create_config = __DIR__ . '/../wp/create_wordpress_config';
			exec( "bash $wp_create_config", $output_config, $wp_result_config );
			$wp_install_check = __DIR__ . '/../wp/install_wordpress_check.sh';
			exec( "bash $wp_install_check", $output, $wp_result );

			if ( 1 === $output ) {
				$wp = __DIR__ . '/../wp/install_wordpress';
				shell_exec( "bash $wp" );
			} elseif ( 1 === $wp_result ) {
				die( 'There was an error. Likely we can not connect to the database' );
			} else {
				$database_name = exec( "wp eval 'echo DB_NAME;'" );
				if ( $database_name ) {
					$database_drop = $io->askConfirmation( "<info>The $database_name database already exists. Can we delete it?</info> " );
				} else {
					$database_drop = $io->askConfirmation( '<info>The database name is undefined. Continue?</info> ' );
				}

				if ( true === $database_drop ) {
					$wp = __DIR__ . '/../wp/install_wordpress';
					exec( "bash $wp", $installoutput, $wpout );
				} else {
					die( 'You did not want to delete the database so we did not begin the WordPress installation.' );
				}
			}
		}
		echo 'finished';
	}

	public static function validatePackage( $args ) {
		if ( 'No' === $args || 'n' === $args ) {
			$args = 'No';
		}
		return $args;
	}

	public static function default_db_name( $string ) {

		$string = self::validate_db_name( $string );

		return $string;
	}

	public static function validate_client_name( $string ) {

		$sanitise = new Sanitise;
		// Limit theme name to 32 characters
		$string = $sanitise->limit_string_length( $string, 64 );
		$string = $sanitise->remove_disallowed_characters( $string );

		return $string;
	}

	public static function validate_theme_name( $string ) {

		$sanitise = new Sanitise;
		// Limit theme name to 32 characters
		$string = $sanitise->limit_string_length( $string, 32 );
		$string = $sanitise->remove_disallowed_characters( $string );

		return $string;
	}

	/**
	 * @param $url
	 *
	 * @return mixed|string
	 */
	public static function validateURLwithoutProtocol( $url ) {
		if ( ! $url || strlen( $url ) < 4 ) {
			return false;
		}
		$sanitise = new Sanitise;
		$url = $sanitise->to_lowercase( $url );
		$url = $sanitise->remove_spaces( $url );
		$url = $sanitise->remove_disallowed_url_protocols( $url );
		$validate = new Validate;
		$url = $validate->validate_url_without_protocol( $url );

		return $url;
	}

	/**
	 * @param $string
	 *
	 * @return mixed
	 */
	public static function validate_lowercase_no_spaces( $string ) {
		$sanitise = new Sanitise;
		$string = $sanitise->to_lowercase( $string );
		$string = $sanitise->remove_spaces( $string );
		$string = $sanitise->remove_disallowed_characters( $string );

		return $string;
	}

	/**
	 * @param $string
	 *
	 * @return mixed
	 */
	public static function validate_lowercase_hyphen_no_spaces( $string ) {
		$sanitise = new Sanitise;
		$string = $sanitise->to_lowercase( $string );
		$string = $sanitise->replace_spaces( $string, '-' );
		$string = $sanitise->remove_disallowed_characters( $string );

		return $string;
	}

	public static function validate_db_username( $string ) {

		$sanitise = new Sanitise;
		// 16 characters for the username
		$string = $sanitise->limit_string_length( $string, 16 );

		$string = $sanitise->to_lowercase( $string );
		$string = $sanitise->remove_spaces( $string );
		$string = $sanitise->remove_disallowed_characters( $string );

		return $string;
	}

	public static function validate_db_name( $string ) {

		// Cannot contain / \ or . characters
		// Cannot contain characters that are not permitted in file names.
		// Cannot end with space characters
		// Permitted characters in unquoted identifiers:
		// ASCII: [0-9,a-z,A-Z$_] (basic Latin letters, digits 0-9, dollar, underscore)
		// Extended: U+0080 .. U+FFFF
		// Permitted characters in quoted identifiers include the full Unicode Basic Multilingual Plane (BMP), except U+0000:
		// ASCII: U+0001 .. U+007F
		// Extended: U+0080 .. U+FFFF
		// ASCII NUL (U+0000) and supplementary characters (U+10000 and higher) are not permitted in quoted or unquoted identifiers.
		// Identifiers may begin with a digit but unless quoted may not consist solely of digits.
		// The identifier quote character is the backtick (“`”):
		// Database, table, and column names cannot end with space characters.
		// The identifier quote character is the backtick (“`”):
		$sanitise = new Sanitise;
		// Length: 64 (NDB storage engine: 63) So we will use 63 for safety.
		$string = $sanitise->limit_string_length( $string, 63 );

		$string = $sanitise->sanitise_with_preg_replace( $string, '/[^0-9,a-z,A-Z$_]/', '' );
		$string = $sanitise->to_lowercase( $string );
		$string = $sanitise->remove_spaces( $string );
		$string = $sanitise->remove_disallowed_characters( $string );

		return $string;
	}

	/* Add db_name and user etc to wordpress config file before WordPress install */

	// public static function showPackages( ShowCommand $show = null ) {
	// new ShowCommand;
	// print_r( $show->getRootRequires() );
	// }
}
